"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unflatten = exports.flatten = void 0;
//@ts-nocheck
// copied from https://github.com/hughsk/flat
function isBuffer(obj) {
    return (obj &&
        obj.constructor &&
        typeof obj.constructor.isBuffer === "function" &&
        obj.constructor.isBuffer(obj));
}
function keyIdentity(key) {
    return key;
}
function flatten(target, opts) {
    opts = opts || {};
    var delimiter = opts.delimiter || ".";
    var maxDepth = opts.maxDepth;
    var transformKey = opts.transformKey || keyIdentity;
    var output = {};
    function step(object, prev, currentDepth) {
        currentDepth = currentDepth || 1;
        Object.keys(object).forEach(function (key) {
            var value = object[key];
            var isarray = opts.safe && Array.isArray(value);
            var type = Object.prototype.toString.call(value);
            var isbuffer = isBuffer(value);
            var isobject = type === "[object Object]" || type === "[object Array]";
            var newKey = prev ? prev + delimiter + transformKey(key) : transformKey(key);
            if (!isarray &&
                !isbuffer &&
                isobject &&
                Object.keys(value).length &&
                (!opts.maxDepth || currentDepth < maxDepth)) {
                return step(value, newKey, currentDepth + 1);
            }
            output[newKey] = value;
        });
    }
    step(target);
    return output;
}
exports.flatten = flatten;
function unflatten(target, opts) {
    opts = opts || {};
    var delimiter = opts.delimiter || ".";
    var overwrite = opts.overwrite || false;
    var transformKey = opts.transformKey || keyIdentity;
    var result = {};
    var isbuffer = isBuffer(target);
    if (isbuffer || Object.prototype.toString.call(target) !== "[object Object]") {
        return target;
    }
    // safely ensure that the key is
    // an integer.
    function getkey(key) {
        var parsedKey = Number(key);
        return isNaN(parsedKey) || key.indexOf(".") !== -1 || opts.object ? key : parsedKey;
    }
    function addKeys(keyPrefix, recipient, target) {
        return Object.keys(target).reduce(function (result, key) {
            result[keyPrefix + delimiter + key] = target[key];
            return result;
        }, recipient);
    }
    function isEmpty(val) {
        var type = Object.prototype.toString.call(val);
        var isArray = type === "[object Array]";
        var isObject = type === "[object Object]";
        if (!val) {
            return true;
        }
        else if (isArray) {
            return !val.length;
        }
        else if (isObject) {
            return !Object.keys(val).length;
        }
    }
    target = Object.keys(target).reduce(function (result, key) {
        var type = Object.prototype.toString.call(target[key]);
        var isObject = type === "[object Object]" || type === "[object Array]";
        if (!isObject || isEmpty(target[key])) {
            result[key] = target[key];
            return result;
        }
        else {
            return addKeys(key, result, flatten(target[key], opts));
        }
    }, {});
    Object.keys(target).forEach(function (key) {
        var split = key.split(delimiter).map(transformKey);
        var key1 = getkey(split.shift());
        var key2 = getkey(split[0]);
        var recipient = result;
        while (key2 !== undefined) {
            if (key1 === "__proto__") {
                return;
            }
            var type = Object.prototype.toString.call(recipient[key1]);
            var isobject = type === "[object Object]" || type === "[object Array]";
            // do not write over falsey, non-undefined values if overwrite is false
            if (!overwrite && !isobject && typeof recipient[key1] !== "undefined") {
                return;
            }
            if ((overwrite && !isobject) || (!overwrite && recipient[key1] == null)) {
                recipient[key1] = typeof key2 === "number" && !opts.object ? [] : {};
            }
            recipient = recipient[key1];
            if (split.length > 0) {
                key1 = getkey(split.shift());
                key2 = getkey(split[0]);
            }
        }
        // unflatten again for 'messy objects'
        recipient[key1] = unflatten(target[key], opts);
    });
    return result;
}
exports.unflatten = unflatten;
//# sourceMappingURL=flat.js.map